@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 1998-2000 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
+    (c) 1998-2005 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,8 @@ static void get_env_as_args(char **argv[], int &argc, char *env);
 static void do_options(int argc, char *argv[],
 		       int &rfm, int &rat, int &org,
 		       int &interactive, int &keep_version, int &user_bufsize,
-		       int &remove_cr, int &show_stats, int &verbose);
+		       int &remove_cr, int &show_stats, int &verbose, 
+		       int &flags, char *protection);
 
 // Start here:
 int main(int argc, char *argv[])
@@ -58,9 +59,12 @@ int main(int argc, char *argv[])
     int   last_infile;
     int   remove_cr = 0;
     int   show_stats = 0;
+    int   printfile = 0;
+    int   flags = 0;
     char  opt;
+    char  protection[255]={'\0'};
     struct timeval start_tv;
-    unsigned long bytes_copied = 0;
+    unsigned long long bytes_copied = 0;
 
     if (argc < 2)
     {
@@ -82,14 +86,14 @@ int main(int argc, char *argv[])
 	do_options(env_argc, env_argv,
 		   rfm, rat,org,
 		   interactive, keep_version, user_bufsize,
-		   remove_cr, show_stats, verbose);
+		   remove_cr, show_stats, verbose, flags, protection);
 
 
 // Parse the command-line options
     do_options(argc, argv,
 	       rfm, rat,org,
 	       interactive, keep_version, user_bufsize,
-	       remove_cr, show_stats, verbose);
+	       remove_cr, show_stats, verbose, flags, protection);
 
     // Work out the buffer size. The default for block transfers is 512
     // bytes unless the user specified otherwise.
@@ -150,7 +154,7 @@ int main(int argc, char *argv[])
     }
 
     // Set up the network links if necessary
-    if (out->setup_link(bufsize, rfm, rat, org))
+    if (out->setup_link(bufsize, rfm, rat, org, flags))
     {
 	out->perror("Error setting up output link");
 	return 1;
@@ -161,7 +165,7 @@ int main(int argc, char *argv[])
 
     for (filenum = optind; filenum < last_infile; filenum++)
     {
-	in = getFile(argv[filenum], verbose);
+	    in = getFile(argv[filenum], verbose);
 
 	// Now we have the first file name, if it is the only input file then
 	// we can check to see if it is a wildcard. If that is so then the
@@ -175,7 +179,7 @@ int main(int argc, char *argv[])
 	    }
 	}
 
-	if (in->setup_link(bufsize, rfm, rat, org))
+	if (in->setup_link(bufsize, rfm, rat, org, flags))
 	{
 	    in->perror("Error setting up input link");
 	    return 1;
@@ -222,6 +226,7 @@ int main(int argc, char *argv[])
 	    {
 		// If the output is a directory so we need to add the
 		// input file's basename to it.
+		out->set_protection(protection);
 		if (out->isdirectory())
 		{
 		    if (out->open(in->get_basename(keep_version), "w+"))
@@ -280,7 +285,14 @@ int main(int argc, char *argv[])
 		    out->perror("Error setting protection");
 		    // Non-fatal error this one.
 		}
-		if (!dntype) out->close();
+		if (!dntype)
+		{
+		    if (out->close())
+		    {
+			out->perror("Error closing output");
+			return 3;
+		    }
+		}
 
 		// Log the operation if we were asked
 		if (verbose && !dntype)
@@ -291,7 +303,11 @@ int main(int argc, char *argv[])
 			   in->get_format_name());
 
 	    }
-	    in->close();
+	    if (in->close())
+	    {
+		out->perror("Error closing input");
+		return 3;
+	    }
 	}
 	while(in->next());
     }
@@ -303,14 +319,14 @@ int main(int argc, char *argv[])
 	long centi_seconds;
 	double rate;
 	double show_secs;
-	
+
 	gettimeofday(&stop_tv, NULL);
 	centi_seconds = (stop_tv.tv_sec - start_tv.tv_sec) * 100 +
 	    (stop_tv.tv_usec - start_tv.tv_usec) / 10000;
 	show_secs = (double)centi_seconds/100.0;
-	
+
 	rate = (double)(bytes_copied/1024) / (double)centi_seconds * 100.0;
-	printf("Sent %ld bytes in %1.2f seconds: %4.2fK/s\n",
+	printf("Sent %lld bytes in %1.2f seconds: %4.2fK/s\n",
 	       bytes_copied, show_secs, rate);
     }
 }
@@ -341,6 +357,9 @@ static void usage(char *name, int dntype, FILE *f)
         fprintf(f, "  -r <fmt>  (s)record format (fix, var, vfc, stm)\n");
         fprintf(f, "  -b <n>       use a block size of <n> bytes\n");
         fprintf(f, "  -d        (s)remove trailing CR on record (DOS file transfer)\n");
+        fprintf(f, "  -l        (r)ignore interlocks on remote file\n");
+	fprintf(f, "  -P        (s)print file to SYS$PRINT\n");
+	fprintf(f, "  -D        (s)delete file on close. Only really useful with -P\n");
         fprintf(f, "  -V           show version number\n");
         fprintf(f, "\n");
         fprintf(f, " (s) - only useful when sending files to VMS\n");
@@ -421,12 +440,13 @@ static void get_env_as_args(char **argv[], int &argc, char *env)
 static void do_options(int argc, char *argv[],
 		       int &rfm, int &rat, int &org,
 		       int &interactive, int &keep_version, int &user_bufsize,
-		       int &remove_cr, int &show_stats, int &verbose)
+		       int &remove_cr, int &show_stats, int &verbose, 
+		       int &flags, char *protection)
 {
     int opt;
     opterr = 0;
     optind = 0;
-    while ((opt=getopt(argc,argv,"?Vvhdr:a:b:kism:")) != EOF)
+    while ((opt=getopt(argc,argv,"?Vvhdr:a:b:kislm:p:PD")) != EOF)
     {
 	switch(opt) {
 	case 'h':
@@ -441,6 +461,10 @@ static void do_options(int argc, char *argv[],
 	    verbose++;
 	    break;
 
+	case 'l':
+	    flags |= file::FILE_FLAGS_RRL;
+	    break;
+
 	case 'r':
 	    if (tolower(optarg[0]) == 'f') rfm = file::RFM_FIX;
 	    if (tolower(optarg[0]) == 's') rfm = file::RFM_STM;
@@ -491,10 +515,32 @@ static void do_options(int argc, char *argv[],
 	    keep_version = TRUE;
 	    break;
 
+	case 'P':
+	    flags |= file::FILE_FLAGS_SPOOL;
+	    break;
+
+	case 'D':
+	    flags |= file::FILE_FLAGS_DELETE;
+	    break;
+
 	case 'b':
 	    user_bufsize = atoi(optarg);
 	    break;
 
+	case 'p':
+	    strcpy(protection, optarg);
+	    strcpy(protection, optarg);
+	    {
+                    // Validate protection
+                    dap_protect_message prot;
+                    if (prot.set_protection(protection) == -1)
+                    {
+                            fprintf(stderr, "Invalid VMS protection string\n");
+                            exit(1);
+                    }
+            }
+	    break;
+
 	case 'd':
 	    remove_cr++;
 	    break;
