@@ -1,6 +1,6 @@
 /******************************************************************************
-    (c) 1998-1999 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
-    
+    (c) 1998-2002 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -45,11 +45,12 @@ dnetfile::dnetfile(char *n, int verbosity):
     isOpen = FALSE;
     verbose = verbosity;
     lasterror = NULL;
+    protection = NULL;
     strcpy(fname, n);
     strcpy(name, n);
 
     // Is this a wildcard filename?
-    if (strchr(name, '*') || 
+    if (strchr(name, '*') ||
 	strchr(name, '%'))
 	wildcard = TRUE;
     else
@@ -61,7 +62,13 @@ dnetfile::~dnetfile()
     dap_close_link();
 }
 
-int dnetfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode)
+void dnetfile::set_protection(char *vmsprot)
+{
+    if (vmsprot[0] != '\0')
+	protection = vmsprot;
+}
+
+int dnetfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode, int flags)
 {
 // If there was a parse error in the file name then fail here
     if (lasterror)
@@ -74,6 +81,7 @@ int dnetfile::setup_link(unsigned int bufsize, int rfm, int rat, int xfer_mode)
     user_rat = rat;
     transfer_mode = xfer_mode;
     user_bufsize = bufsize;
+    user_flags = flags;
 
     struct accessdata_dn accessdata;
     memset(&accessdata, 0, sizeof(accessdata));
@@ -112,7 +120,7 @@ int dnetfile::open(char *filename,char *mode)
     return open(mode);
 }
 
-// Open a file already named 
+// Open a file already named
 int dnetfile::open(char *mode)
 {
     int real_rfm, real_rat;
@@ -140,7 +148,7 @@ int dnetfile::open(char *mode)
 	if (writing)
 	{
 	    status = dap_send_attributes();
-	    
+
 	    // Send default file name
 	    if (wildcard)
 	    {
@@ -169,7 +177,7 @@ int dnetfile::open(char *mode)
 // Use the file's real attributes if the user just wants defaults.
 	    if (user_rfm == RFM_DEFAULT)  file_rfm = real_rfm;
 	    if (user_rat == RAT_DEFAULT)  file_rat = real_rat;
-	    
+
 // Get the file's real name in case the user specified a wildcard
 	    strcpy(name, filname);
 	}
@@ -205,12 +213,12 @@ int dnetfile::read(char *buf, int len)
 
 // If the file has implied carriage control then add an LF to the end of the
 // line
-    if ((file_rfm != RFM_STMLF) && 
+    if ((file_rfm != RFM_STMLF) &&
 	(file_rat & RAT_CR || file_rat & RAT_PRN))
 	buf[retlen++] = '\n';
 
 // Print files have a two-byte header indicating the line length.
-    if (file_rat & RAT_PRN) 
+    if (file_rat & RAT_PRN)
     {
 	memmove(buf, buf+file_fsz, retlen-file_fsz);
 	retlen -= file_fsz;
@@ -219,13 +227,13 @@ int dnetfile::read(char *buf, int len)
 // FORTRAN files have a leading character that indicates carriage control
     if (file_rat & RAT_FTN)
     {
-	switch (buf[0]) 
+	switch (buf[0])
 	{
-	  
+
 	case '+': // No new line
 	  buf[0] = '\r';
 	  break;
-	  
+
 	case '1': // Form Feed
 	  buf[0] = '\f';
 	  break;
@@ -259,7 +267,7 @@ int dnetfile::next()
     int status;
     int real_rfm, real_rat;
 
-    if (!wildcard) 
+    if (!wildcard)
     {
 	return FALSE;
     }
@@ -330,7 +338,7 @@ void dnetfile::make_basename(int keep_version)
     char        *start;
     char        *end;
     unsigned int i;
-		    
+
     /* Find the start of the name */
     start = rindex(name, ']');
     if (!start) start = rindex(name, ':');
@@ -356,9 +364,9 @@ void dnetfile::make_basename(int keep_version)
 char *dnetfile::get_printname(char *filename)
 {
     static char pname[1024];
-    
+
     strcpy(pname, node);
-    if (*user) 
+    if (*user)
     {
 	strcat(pname, "\"");
 	strcat(pname, user);
@@ -396,7 +404,7 @@ char *dnetfile::get_format_name()
 	{
 	case RAT_NONE:
 	    return "blocks";
-	    
+
 	default:
 	    return "records";
 	}
