@@ -1,5 +1,5 @@
 /******************************************************************************
-    (c) 1998-1999 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
+    (c) 1998-2006 P.J. Caulfield               patrick@tykepenguin.cix.co.uk
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,29 +69,45 @@ int dnetfile::dap_send_access()
     dap_access_message acc;
     acc.set_accfunc(dap_access_message::OPEN);
 
+    conn.set_blocked(true);
+
     if (transfer_mode == MODE_BLOCK)
     {
-	acc.set_fac( (1<<dap_access_message::FB$GET) ||
-		     (1<<dap_access_message::FB$BIO) );
-	acc.set_shr(1<<dap_access_message::FB$BRO); // No sharing PJC CHECK
+	acc.set_fac( (1<<dap_access_message::FB$GET) |
+		     (1<<dap_access_message::FB$BRO));
+	acc.set_shr(1<<dap_access_message::FB$GET);
     }
 
     if (writing)
     {
 	acc.set_accfunc(dap_access_message::CREATE);
 	acc.set_fac(dap_access_message::FB$PUT);
-	acc.set_shr(1<<dap_access_message::FB$BRO); // No sharing PJC CHECK
+	acc.set_shr(1<<dap_access_message::FB$BRO);
     }
     else // We build our own attributes message when writing.
     {
 	dap_attrib_message att;
+
+	att.set_fop_bit(dap_attrib_message::FB$SQO);
+	att.set_org(dap_attrib_message::FB$SEQ);
+	att.set_rfm(dap_attrib_message::FB$VAR);
+	att.set_mrs(0);
+	att.set_datatype(dap_attrib_message::IMAGE);
+	if (user_flags & FILE_FLAGS_RRL) {
+	    acc.set_shr(1<<dap_access_message::FB$PUT |
+	                1<<dap_access_message::FB$UPD |
+	                1<<dap_access_message::FB$GET |
+	                1<<dap_access_message::FB$DEL);
+	}
+
 	att.write(conn);
     }
     acc.set_display(dap_access_message::DISPLAY_MAIN_MASK |
 		    dap_access_message::DISPLAY_PROT_MASK |
 		    dap_access_message::DISPLAY_NAME_MASK);
     acc.set_filespec(filname);
-    return !acc.write(conn);
+    acc.write(conn);
+    return !conn.set_blocked(false);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -206,6 +222,10 @@ int dnetfile::dap_send_get_or_put()
     {
 	ctl.set_ctlfunc(dap_control_message::PUT);
     }
+    if (user_flags & file::FILE_FLAGS_RRL)
+    {
+    	ctl.set_rop_bit(dap_control_message::RB$RRL);
+    }
     return !ctl.write(conn);
 }
 /*-------------------------------------------------------------------------*/
@@ -215,8 +235,14 @@ int dnetfile::dap_send_accomp()
 {
     dap_accomp_message accomp;
     accomp.set_cmpfunc(dap_accomp_message::CLOSE);
+
+    /* Remote printing support */
+    if (user_flags & FILE_FLAGS_SPOOL)
+	    accomp.set_fop_bit(dap_attrib_message::FB$SPL);
+    if (user_flags & FILE_FLAGS_DELETE)
+	    accomp.set_fop_bit(dap_attrib_message::FB$DLT);
     if (!accomp.write(conn)) return -1;
-    conn.set_blocked(false); //PJC
+    conn.set_blocked(false);
 
     if (writing)
 	return dap_get_reply();
@@ -239,6 +265,9 @@ int dnetfile::dap_get_record(char *rec, int reclen)
 		ateof = TRUE;
 		return -1;
 	    }
+	    // good STATUS here means we need to request the next record
+	    if (dap_check_status(m, 0) == 0)
+		dap_send_get_or_put(); 
 	    return dap_check_status(m,0);
 	}
 
@@ -264,7 +293,7 @@ int dnetfile::dap_get_record(char *rec, int reclen)
     return -1;
 }
 
-// Send a record to VMS
+// Send a record to the other end
 int dnetfile::dap_put_record(char *rec, int reclen)
 {
     if (verbose > 2) DAPLOG((LOG_INFO, "in dap_put_record(%d bytes)\n", reclen));
@@ -272,7 +301,6 @@ int dnetfile::dap_put_record(char *rec, int reclen)
     conn.set_blocked(true);
 
     dap_data_message data;
-
     data.set_data(rec, reclen);
     data.write_with_len(conn);
 
@@ -291,9 +319,13 @@ int dnetfile::dap_send_attributes()
     dap_attrib_message att;
     dap_alloc_message  all;
 
-
     att.set_org(dap_attrib_message::FB$SEQ);
-    att.set_rfm(dap_attrib_message::FB$STMLF);
+
+// Only VMS can cope with StreamLF files.
+    if (conn.get_remote_os() == 7)
+        att.set_rfm(dap_attrib_message::FB$STMLF);
+    else
+        att.set_rfm(dap_attrib_message::FB$VAR);
     att.set_bls(512);
     att.set_mrs(user_bufsize);
 
@@ -316,7 +348,19 @@ int dnetfile::dap_send_attributes()
 
     conn.set_blocked(true);
     att.write(conn);
-    all.write(conn);
+
+// VMS likes an ALLOC message
+    if (conn.get_remote_os() == 7)
+	all.write(conn);
+
+    if (protection)
+    {
+	dap_protect_message prot;
+	if (prot.set_protection(protection) == -1)
+	    fprintf(stderr, "Error in protection string '%s' - not sent\n", protection);
+	else
+	    prot.write(conn);
+    }
     return conn.set_blocked(false);
 }
 
@@ -348,10 +392,15 @@ int dnetfile::dap_check_status(dap_message *m, int status)
 
     // Save this stuff so we can delete the message
     int code = sm->get_code() & 0xFF;
+    int maccode = sm->get_code() >> 12;
     char *err = sm->get_message();
 
+    if (verbose > 1)
+	DAPLOG((LOG_INFO, "dap_check_status. maccode=%d code: octal: %o (hex: %x)\n", maccode, sm->get_code(),sm->get_code()));
+
     delete m;
 
+    if (maccode == 1) return status; // Success
     if (code == 0225) return status; // Success
     if (code == 047)  return code;   // EOF
     lasterror = err;
