@@ -16,6 +16,7 @@ class dap_item
  public:
     virtual bool read(dap_connection&) = 0;
     virtual bool write(dap_connection&) = 0;
+    virtual ~dap_item() {}
 };
 
 class dap_bytes : public dap_item // number of bytes
@@ -27,12 +28,12 @@ class dap_bytes : public dap_item // number of bytes
 	    value = (unsigned char *)malloc(size+1); // May be a string
 	    memset(value, 0, size);
 	}
-    
+
     virtual ~dap_bytes()
 	{
 	    free(value);
 	}
-     
+
     unsigned char  get_byte(int bytenum);
     char          *get_string();
     unsigned short get_short();
@@ -42,10 +43,11 @@ class dap_bytes : public dap_item // number of bytes
     void set_short(unsigned short newval);
     void set_int(unsigned int newval);
     void set_string(const char *newval);
-    
+    void set_value(const char *newval, int len);
+
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
-    
+
  private:
     int            length;
     unsigned char *value;
@@ -62,7 +64,7 @@ class dap_ex : public dap_item // EX extensible field
 	    value = (unsigned char *)malloc(size);
 	    memset(value, 0, size);
 	}
-    
+
     virtual ~dap_ex()
 	{
 	    free(value);
@@ -77,7 +79,7 @@ class dap_ex : public dap_item // EX extensible field
     void          clear_bit(int bit);
     void          set_byte(int bytenum, unsigned char newval);
     void          clear_all() {memset(value, 0, length);}
-    
+
  private:
     unsigned char length;
     unsigned char real_length;
@@ -110,9 +112,10 @@ class dap_image : public dap_item // "I" field
     unsigned int   get_int();
     unsigned char  get_length() {return real_length;};
     void           set_string(const char *);
+    void           set_value(const char *, int);
     void           set_int(unsigned int v);
     void           set_short(unsigned short v);
-    
+
  private:
     unsigned char  length;
     unsigned char  real_length;
@@ -123,7 +126,7 @@ class dap_image : public dap_item // "I" field
 class dap_message
 {
  public:
-    dap_message() {};
+    dap_message() {}
     virtual ~dap_message() {};
 
     virtual bool read(dap_connection&)=0;
@@ -135,7 +138,7 @@ class dap_message
     unsigned char get_type();
     char *type_name();
     static char *type_name(int);
-    
+
     // Message Types;
     static const unsigned char CONFIG  = 1;
     static const unsigned char ATTRIB  = 2;
@@ -153,14 +156,13 @@ class dap_message
     static const unsigned char PROTECT = 14;
     static const unsigned char NAME    = 15;
     static const unsigned char ACL     = 16;
-    
+
 
  protected:
     unsigned char msg_type;
     int           length;
     unsigned char flags;
 
-
     int send_header(dap_connection &c);
     int send_long_header(dap_connection &c);
     int send_header(dap_connection &c, bool);
@@ -207,6 +209,10 @@ class dap_config_message: public dap_message
 	{
 	    return ostype.get_byte(0);
 	}
+    bool get_syscap_bit(int bit)
+	{
+		return syscap.get_bit(bit);
+	}
 
     // OSs
     static const int OS_ILLEGAL =  0;
@@ -223,7 +229,20 @@ class dap_config_message: public dap_message
     static const int OS_OS8     = 11;
     static const int OS_RSX11MP = 12;
     static const int OS_COPOS11 = 13;
-    
+    static const int OS_POS     = 14;
+    static const int OS_VAXLELN = 15;
+    static const int OS_CPM     = 16;
+    static const int OS_MSDOS   = 17;
+    static const int OS_ULTRIX  = 18;
+    static const int OS_ULTRIX11 = 19;
+    static const int OS_DTF_MVS = 21;
+    static const int OS_MACOS   = 22;
+    static const int OS_OS2     = 23;
+    static const int OS_DTF_VM  = 24;
+    static const int OS_OSF1    = 25;
+    static const int OS_WIN_NT  = 26;
+    static const int OS_WIN_95  = 27;
+
  private:
     dap_bytes bufsiz;
     dap_bytes ostype;
@@ -259,12 +278,14 @@ class dap_attrib_message: public dap_message
 	hbk(5),
 	ebk(5),
 	ffb(2),
-	sbn(5)
+        sbn(5),
+        jnl(4)
 	{msg_type = ATTRIB;}
 
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
 
+    int get_menu_bit(int);
     int get_datatype();
     int get_org();
     int get_rfm();
@@ -286,6 +307,7 @@ class dap_attrib_message: public dap_message
     int get_ebk();
     int get_ffb();
     int get_sbn();
+    int get_jnl();
 
     void set_datatype(int);
     void set_org(int);
@@ -317,6 +339,30 @@ class dap_attrib_message: public dap_message
     void set_file(const char *file, bool show_dev);
     unsigned long get_size();
 
+    // Bits in ATTMENU
+    static const int MENU_DATATYPE = 0;
+    static const int MENU_ORG = 1;
+    static const int MENU_RFM = 2;
+    static const int MENU_RAT = 3;
+    static const int MENU_BLS = 4;
+    static const int MENU_MRS = 5;
+    static const int MENU_ALQ = 6;
+    static const int MENU_BKS = 7;
+    static const int MENU_FSZ = 8;
+    static const int MENU_MRN = 9;
+    static const int MENU_RUNSYS = 10;
+    static const int MENU_DEQ = 11;
+    static const int MENU_FOP = 12;
+    static const int MENU_BSZ = 13;
+    static const int MENU_DEV = 14;
+    static const int MENU_SDC = 15;
+    static const int MENU_LRL = 16;
+    static const int MENU_HBK = 17;
+    static const int MENU_EBK = 18;
+    static const int MENU_FFB = 19;
+    static const int MENU_SBN = 20;
+    static const int MENU_JNL = 21;
+
     //DATATYPEs
     static const int ASCII      = 0;
     static const int IMAGE      = 1;
@@ -347,12 +393,14 @@ class dap_attrib_message: public dap_message
     static const int FB$PRN = 2;
     static const int FB$BLK = 3;
     static const int FB$LSA = 6;
+    static const int FB$MACY11 = 7;
 
     //FOPs:
     static const int FB$RWO = 0;
     static const int FB$RWC = 1;
     static const int FB$POS = 3;
     static const int FB$DLK = 4;
+    static const int FB$DIR = 5;
     static const int LOCKED = 6;
     static const int FB$CTG = 7;
     static const int FB$SUP = 8;
@@ -402,7 +450,7 @@ class dap_attrib_message: public dap_message
     static const int FOREIGN = 23;
     static const int NETWORK_DEVICE = 24;
     static const int GENERIC_DEVICE = 25;
-    
+
  private:
     dap_ex    attmenu;
     dap_ex    datatype;
@@ -426,6 +474,7 @@ class dap_attrib_message: public dap_message
     dap_image ebk;
     dap_bytes ffb;
     dap_image sbn;
+    dap_ex    jnl;
 };
 
 //ACCESS message TYPE=3
@@ -451,7 +500,7 @@ class dap_access_message: public dap_message
     void set_fac(int);
     void set_shr(int);
     void set_display(int);
-    
+
     int   get_accfunc();
     int   get_accopt();
     char *get_filespec();
@@ -475,6 +524,8 @@ class dap_access_message: public dap_message
     static const int FB$TRN = 4;
     static const int FB$BIO = 5;
     static const int FB$BRO = 6;
+    static const int FB$APP = 7; // FAC (Append)
+    static const int FB$NIL = 7; // SHR (no access by other users)
 
     // bits for DISPLAY
     static const int DISPLAY_MAIN    = 0;
@@ -485,6 +536,10 @@ class dap_access_message: public dap_message
     static const int DISPLAY_PROT    = 5;
     static const int DISPLAY_ACL     = 7;
     static const int DISPLAY_NAME    = 8;
+    static const int DISPLAY_3PTNAME = 9;
+    static const int DISPLAY_COLLTBL = 10;
+    static const int DISPLAY_CDA     = 11;
+    static const int DISPLAY_TCL     = 12;
 
     // masks for DISPLAY
     static const int DISPLAY_MAIN_MASK    =   1;
@@ -495,7 +550,11 @@ class dap_access_message: public dap_message
     static const int DISPLAY_PROT_MASK    =  32;
     static const int DISPLAY_ACL_MASK     = 128;
     static const int DISPLAY_NAME_MASK    = 256;
-    
+    static const int DISPLAY_3PTNAME_MASK = 512;
+    static const int DISPLAY_COLLTBL_MASK = 1024;
+    static const int DISPLAY_CDA_MASK    = 2048;
+    static const int DISPLAY_TCL_MASK    = 4096;
+
  private:
     dap_bytes accfunc;
     dap_ex    accopt;
@@ -518,19 +577,24 @@ class dap_control_message: public dap_message
 	krf(1),
         rop(6),
 	hsh(5),
-        display(4)
+        display(4),
+	blkcnt(1),
+	usz(2)
 	{msg_type = CONTROL;}
-    
+
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
 
     void set_ctlfunc(int f);
     void set_rac(int r);
     void set_key(const char *k);
+    void set_key(const char *k, int l);
     void set_krf(int f);
     void set_rop_bit(int f);
     void set_rop(int f);
     void set_display(int);
+    void set_blkcnt(int);
+    void set_usz(int);
 
     int   get_ctlfunc();
     int   get_rac();
@@ -539,7 +603,9 @@ class dap_control_message: public dap_message
     bool  get_rop_bit(int f);
     int   get_display();
     unsigned long get_long_key();
-    
+    int   get_blkcnt();
+    int   get_usz();
+
     // ctlfunc - Control functions:
     static const int GET            = 1;
     static const int CONNECT        = 2;
@@ -585,8 +651,14 @@ class dap_control_message: public dap_message
     static const int RB$NLK  = 11;
     static const int RB$RLK  = 12;
     static const int RB$BIO  = 13;
-    static const int RB$NXR  = 14;
-    
+    static const int RB$LIM  = 14;
+    static const int RB$NXR  = 15;
+    static const int RB$WAT  = 16;
+    static const int RB$RRL  = 17;
+    static const int RB$REA  = 18;
+    static const int RB$KLE  = 19;
+    static const int RB$KLT  = 20;
+
  private:
     dap_bytes ctlfunc;
     dap_ex    ctlmenu;
@@ -596,6 +668,8 @@ class dap_control_message: public dap_message
     dap_ex    rop;
     dap_image hsh;
     dap_ex    display;
+    dap_bytes blkcnt;
+    dap_bytes usz;
 };
 
 //CONTRAN (Continue Transfer) message TYPE=5
@@ -613,10 +687,12 @@ class dap_contran_message: public dap_message
     static const int SKIP      = 2;
     static const int ABORT     = 3;
     static const int RESUME    = 4;
+    static const int TERMINATE = 5;
+    static const int RESYNC    = 6;
 
     int get_confunc();
     void set_confunc(int f);
-    
+
  private:
     dap_bytes confunc;
 };
@@ -661,7 +737,10 @@ class dap_accomp_message: public dap_message
     static const int PURGE         = 3;
     static const int END_OF_STREAM = 4;
     static const int SKIP          = 5;
-    
+    static const int CHANGE_BEGIN  = 6;
+    static const int CHANGE_END    = 7;
+    static const int TERMINATE     = 8;
+
  private:
     dap_bytes cmpfunc;
     dap_ex    fop;
@@ -682,19 +761,19 @@ class dap_data_message: public dap_message
 	{msg_type = DATA;}
 
     ~dap_data_message();
-    
+
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
     virtual bool write_with_len(dap_connection&);
     virtual bool write_with_len256(dap_connection&);
-    
+
     int   get_recnum();
     int   get_datalen();
     char *get_dataptr();
     void  set_recnum(int r);
     void  get_data(char *, int *);
     void  set_data(const char *, int);
-    
+
  private:
     dap_image  recnum;
     char      *data;
@@ -724,7 +803,7 @@ class dap_status_message: public dap_message
     void  set_rfa(long);
     char *get_message();
 
-    
+
  private:
     dap_bytes stscode;
     dap_image rfa;
@@ -760,11 +839,11 @@ class dap_key_message: public dap_message
 	tks(1),
 	mrl(2)
 	{msg_type = KEYDEF;}
-	
+
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
     ~dap_key_message();
-    
+
     // Flags - bit positions
     static const unsigned int XB$DUP = 1;
     static const unsigned int XB$CHG = 2;
@@ -779,8 +858,8 @@ class dap_key_message: public dap_message
 
     void set_knm(char *);
     void set_ref(int);
-    
-    
+
+
  private:
     dap_ex keymenu;
     dap_ex flg;
@@ -825,7 +904,7 @@ class dap_alloc_message: public dap_message
 
     virtual bool read(dap_connection&);
     virtual bool write(dap_connection&);
-    
+
  private:
     dap_ex    allmenu;
     dap_bytes vol;
@@ -858,7 +937,7 @@ class dap_summary_message: public dap_message
     int get_noa();
     int get_nor();
     int get_pvn();
-    
+
  private:
     dap_ex    summenu;
     dap_bytes nok;
@@ -877,7 +956,8 @@ class dap_date_message: public dap_message
 	rdt(18),
 	edt(18),
 	rvn(2),
-	bdt(18)
+	bdt(18),
+	udt(18) // Ultrix date
 	{msg_type = DATE;}
 
     virtual bool read(dap_connection&);
@@ -887,11 +967,13 @@ class dap_date_message: public dap_message
     char *get_rdt();
     char *get_edt();
     char *get_bdt();
+    char *get_udt();
 
     time_t get_cdt_time();
     time_t get_rdt_time();
     time_t get_edt_time();
     time_t get_bdt_time();
+    time_t get_udt_time();
     int    get_rvn();
 
     void set_cdt(time_t);
@@ -902,20 +984,21 @@ class dap_date_message: public dap_message
     void set_rvn(int);
 
     char *make_y2k(char *dt);
-    
+
  private:
     dap_ex    datmenu;
     dap_bytes cdt;
     dap_bytes rdt;
     dap_bytes edt;
     dap_bytes rvn;
-    dap_bytes bdt; // backup date is not mentinoed in the DAP spec but
+    dap_bytes bdt; // backup date is not mentioned in the DAP spec but
                    // it exists in real life I promise you.
+    dap_bytes udt; // Gah! another one. This one from Ultrix
 
     char *time_to_string(time_t);
     time_t string_to_time_t(const char *);
 
-    
+
     static char *months[];
     static const int NUM_MONTHS = 12;
 };
@@ -940,11 +1023,12 @@ class dap_protect_message: public dap_message
     void set_owner(gid_t, uid_t); // Converts GID/UID to names
     void set_owner(const char *); // Any old name will do here.
     void set_protection(mode_t);
+    int  set_protection(char *);
 
     char    *get_owner();
     char    *get_protection(); // In VMS format
     mode_t   get_mode();       // In Unix format
-    
+
  private:
     dap_ex    protmenu;
     dap_image owner;
@@ -979,7 +1063,7 @@ class dap_name_message: public dap_message
     static const int FILENAME  = 2;
     static const int DIRECTORY = 4;
     static const int VOLUME    = 8;
-    
+
  private:
     dap_ex    nametype;
     dap_image namespec;
