@@ -13,7 +13,10 @@
 int main(int argc, char *argv[])
 {
     /* Open the file. Keep the RMSHANDLE returned */
-    RMSHANDLE h = rms_open("marsha::index.dat", O_RDWR, NULL);
+    RMSHANDLE h;
+
+    h = rms_open("ford::index.dat", O_RDWR, NULL);
+
     if (h)
     {
         char b[10240];
